import json
import requests
from HQApi.exceptions import ApiResponseError, BannedIPError


class BaseHQApi:
    def __init__(self, authtoken):
        self.authtoken = authtoken

    def api(self):
        return self

    def get_users_me(self):
        return self.fetch("GET", "users/me")

    def get_user(self, id: str):
        return self.fetch("GET", "users/{}".format(id))

    def search(self, name):
        return self.fetch("GET", 'users?q={}'.format(name))

    def get_payouts_me(self):
        return self.fetch("GET", "users/me/payouts")

    def get_show(self):
        return self.fetch("GET", "shows/now")

    def easter_egg(self, type: str = "makeItRain"):
        return self.fetch("POST", "easter-eggs/{}".format(type))

    def make_payout(self, email: str):
        return self.fetch("POST", "users/me/payouts", {"email": email})

    def send_code(self, phone: str, method: str = "sms"):
        return self.fetch("POST", "verifications", {"phone": phone, "method": method})

    def confirm_code(self, verificationid: str, code: int):
        return self.fetch("POST", "verifications/{}".format(verificationid), {"code": code})

    def register(self, verificationid: str, name: str, referral: str = None):
        return self.fetch("POST", "users", {
            "country": "MQ==", "language": "eu",
            "referringUsername": referral,
            "username": name,
            "verificationId": verificationid})

    def aws_credentials(self):
        return self.fetch("GET", "credentials/s3")

    def delete_avatar(self):
        return self.fetch("DELETE", "users/me/avatarUrl")

    def add_friend(self, id: str):
        return self.fetch("POST", "friends/{}/requests".format(id))

    def friend_status(self, id: str):
        return self.fetch("GET", "friends/{}/status".format(id))

    def remove_friend(self, id: str):
        return self.fetch("DELETE", "friends/{}".format(id))

    def accept_friend(self, id: str):
        return self.fetch("PUT", "friends/{}/status".format(id), {"status": "ACCEPTED"})

    def check_username(self, name: str):
        return self.fetch("POST", "usernames/available", {"username": name})

    def custom(self, method, func, data):
        return self.fetch(method, func, data)


class HQApi(BaseHQApi):
    def __init__(self, authtoken: str = "", version: str = "1.30.0", proxy: str = None):
        super().__init__(authtoken)
        self.authToken = authtoken
        self.version = version
        if authtoken == "":
            self.headers = {
                "x-hq-client": "Android/" + self.version}
        else:
            self.headers = {
                "Authorization": "Bearer " + self.authtoken,
                "x-hq-client": "Android/" + self.version}
        self.p = dict(http=proxy, https=proxy)

    def fetch(self, method="GET", func="", data=None):
        if data is None:
            data = {}
        try:
            if method == "GET":
                content = requests.get("https://api-quiz.hype.space/{}".format(func), data=data,
                                       headers=self.headers, proxies=self.p).json()
            elif method == "POST":
                content = requests.post("https://api-quiz.hype.space/{}".format(func), data=data,
                                        headers=self.headers, proxies=self.p).json()
            elif method == "PATCH":
                content = requests.patch("https://api-quiz.hype.space/{}".format(func), data=data,
                                         headers=self.headers, proxies=self.p).json()
            elif method == "DELETE":
                content = requests.delete("https://api-quiz.hype.space/{}".format(func), data=data,
                                          headers=self.headers, proxies=self.p).json()
            elif method == "PUT":
                content = requests.put("https://api-quiz.hype.space/{}".format(func), data=data,
                                       headers=self.headers, proxies=self.p).json()
            else:
                content = requests.get("https://api-quiz.hype.space/{}".format(func), data=data,
                                       headers=self.headers, proxies=self.p).json()
            error = content.get("error")
            if error:
                raise ApiResponseError(json.dumps(content))
            return content
        except json.decoder.JSONDecodeError:
            raise BannedIPError("Your IP is banned")



import json
import re
import threading
from lomond import WebSocket
from lomond.persist import persist
from HQApi import HQApi
from HQApi.exceptions import NotLive, WebSocketNotAvailable, ApiResponseError, BannedIPError


class HQWebSocket:
    def __init__(self, api: HQApi, demo: bool = False, log_new_methods: bool = True, proxy: str = None):
        self.api = api
        self.log = log_new_methods
        if self.log and not demo:
            print("[HQApi] Thanks for contributing HQApi methods! Please upload log file to https://github.com/katant/hqapi/issues.")
        self.handlers = {}
        self.authtoken = self.api.authtoken
        self.headers = self.api.headers
        self.use_demo = False
        try:
            self.headers["Authorization"]
        except:
            if demo:
                self.use_demo = True
            else:
                raise WebSocketNotAvailable("You can't use websocket without bearer")
        try:
            self.show = HQApi.get_show(api)
            self.socket = self.show["broadcast"]["socketUrl"].replace("https", "wss")
            self.broadcast = self.show['broadcast']['broadcastId']
        except BannedIPError or ApiResponseError:
            if demo:
                self.use_demo = True
            else:
                raise WebSocketNotAvailable("You can't use websocket with banned IP or invalid auth")
        except:
            if demo:
                self.use_demo = True
            else:
                raise NotLive("Show isn't live and demo mode is disabled")
        if self.use_demo:
            print("[HQApi] Using demo websocket! Don't create issues with this websocket")
            self.socket = "wss://hqecho.herokuapp.com"  # Websocket with questions 24/7
            self.broadcast = 1
            self.ws = WebSocket(self.socket)
        else:
            self.ws = WebSocket(self.socket)
            for header, value in self.headers.items():
                self.ws.add_header(str.encode(header), str.encode(value))

    def call(self, data):
        if data["type"] in self.handlers:
            for h in self.handlers[data["type"]]:
                h(data["data"])

    def event(self, type):
        def registerhandler(handler):
            if type in self.handlers:
                self.handlers[type].append(handler)
            else:
                self.handlers[type] = [handler]
            return handler
        return registerhandler

    def listen(self):
        thread = HQWebsocketListener(self)
        thread.start()

    def send_json(self, json=None):
        if json is None:
            json = {}
        self.ws.send_json(json)

    def send_life(self, questionId: int):
        self.send_json({"questionId": questionId, "authToken": self.authtoken,
                        "broadcastId": self.broadcast,
                        "type": "useExtraLife"})

    def send_answer(self, answerId: int, questionId: int):
        self.send_json({"answerId": answerId,
                        "questionId": questionId, "authToken": self.authtoken,
                        "broadcastId": self.broadcast, "type": "answer"})

    def send_comment(self, avatarUrl: str, message: str, userId: int, username: str):
        self.send_json({"metadata": {"avatarUrl": avatarUrl,
                                     "interaction": "chat", "message": message, "userId": userId,
                                     "username": username}, "itemId": "chat",
                        "authToken": self.authtoken,
                        "broadcastId": self.broadcast, "type": "interaction"})

    def send_wheel(self, showId: int, letter: str):
        self.send_json({"type": "spin", "authToken": self.authtoken,
                        "showId": showId, "broadcastId": self.broadcast,
                        "letter": letter})

    def send_letter(self, showId: int, letter: str, roundId: int):
        self.send_json({"type": "guess", "authToken": self.authtoken, "showId": showId,
                        "broadcastId": self.broadcast, "letter": letter,
                        "roundId": roundId})

    def get_erasers(self, friendsIds: dict):
        self.send_json({"authToken": self.authtoken,
                        "friendsIds": friendsIds,
                        "broadcastId": self.broadcast,
                        "type": "erase1Earned"})

    def send_eraser(self, questionId: int):
        self.send_json({"type": "erase1",
                        "questionId": questionId,
                        "authToken": self.authtoken,
                        "broadcastId": self.broadcast})

    def get(self):
        return self.ws

    def close(self):
        self.ws.close()


class HQWebsocketListener(threading.Thread):
    def __init__(self, new):
        threading.Thread.__init__(self)
        self.new = new
        self.methods = ["broadcastStats", "gameStatus", "interaction", "question", "questionClosed", "questionSummary", "questionFinished"]

    def run(self):
        for msg in persist(self.new.ws):
            if msg.name == "text":
                data = json.loads(re.sub(r"[\x00-\x1f\x7f-\x9f]", "", msg.text))
                if self.new.log and data["type"] not in self.methods:
                    print("[HQApi] New %s method detected, json: %s" % (data["type"], data))
                    self.methods.append(data["type"])
                self.new.call({"type": data["type"], "data": data})



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="HQApi",
    version="1.12.0",
    author="Katant",
    author_email="katant.savelev@yandex.ru",
    description="HQ Trivia & Words API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/katant/HQApi",
    install_requires=['requests', 'lomond'],
    packages=["HQApi"],
    python_requires='>=3.4',
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
)
